package org.artifactory.lock.service;

import org.artifactory.concurrent.NonStandardLockCategory;
import org.artifactory.lock.util.ReleaseLockStrategyUtils;
import org.artifactory.storage.db.locks.service.DbLocksService;
import org.artifactory.storage.db.servers.service.ArtifactoryServersCommonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * @author Mor Merhav
 */
@Component
public class NonStandardReleaseLockStrategy implements ReleaseLockStrategy {

    private final DbLocksService dbLocksService;
    private final ArtifactoryServersCommonService serversCommonService;

    @Autowired
    public NonStandardReleaseLockStrategy(DbLocksService dbLocksService,
            ArtifactoryServersCommonService serversCommonService) {
        this.dbLocksService = dbLocksService;
        this.serversCommonService = serversCommonService;
    }

    @Override
    public void releaseOrphaned() {
        NonStandardLockCategory[] nonStandardLockCategories = NonStandardLockCategory.values();
        for (NonStandardLockCategory nonStandardLockCategory : nonStandardLockCategories) {
            long orphanLockGraceTimeSeconds = TimeUnit.MINUTES
                    .toSeconds(nonStandardLockCategory.getOrphanLockGraceTimeMinutes());
            Set<String> activeServers = ReleaseLockStrategyUtils
                    .findActiveServers(serversCommonService, orphanLockGraceTimeSeconds);
            dbLocksService
                    .releaseOrphanLocksByCategoryExcludingServers(activeServers, nonStandardLockCategory.getName());
        }
    }

    @Override
    public void releaseExpired() {
        NonStandardLockCategory[] nonStandardLockCategories = NonStandardLockCategory.values();
        for (NonStandardLockCategory nonStandardLockCategory : nonStandardLockCategories) {
            if (nonStandardLockCategory.isExpirable()) {
                dbLocksService.releaseExpiredLocksByCategory(nonStandardLockCategory.getName(),
                        nonStandardLockCategory.getLockLeaseTimeMinutes());
            }
        }
    }
}
