package org.artifactory.metrics.framework;

import com.jfrog.sysconf.SysConfig;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.repo.http.HttpConnectionPoolMonitorService;
import org.artifactory.security.access.AccessService;
import org.artifactory.storage.binstore.service.GarbageCollectionDatastoreService;
import org.artifactory.storage.db.util.JdbcHelper;
import org.artifactory.storage.service.StorageMetricCollector;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.artifactory.test.TestUtils;
import org.mockito.Mock;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import java.util.Optional;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertNull;
import static org.testng.Assert.assertTrue;

public class MetricsServiceImplTest extends ArtifactoryHomeBoundTest {

    @Mock
    private AccessService accessService;
    @Mock
    private GarbageCollectionDatastoreService garbageCollectionDatastoreService;
    @Mock
    private HttpConnectionPoolMonitorService httpConnectionPoolMonitorService;
    @Mock
    private StorageMetricCollector storageMetricCollector;
    @Mock
    private JdbcHelper jdbcHelper;
    @Mock
    private CentralConfigService configService;

    MetricsServiceImpl metricsService;

    @BeforeClass
    public void setup() {
        metricsService =
                new MetricsServiceImpl(accessService, garbageCollectionDatastoreService, jdbcHelper,
                        httpConnectionPoolMonitorService, storageMetricCollector, configService);
    }

    @Test
    public void testMetricsService() {
        SysConfig sysConfig = mock(SysConfig.class);
        SysConfig sysConfigBackUp = TestUtils.getField(ArtifactoryHome.get(), "sysConfig", SysConfig.class);
        TestUtils.setField(ArtifactoryHome.get(), "sysConfig", sysConfig);

        when(sysConfig.get("artifactory.database.type", "derby")).thenReturn("derby");
        when(sysConfig.getBoolean("artifactory.metrics.enabled", false)).thenReturn(false);
        metricsService.onContextCreated();
        assertFalse(metricsService.isMetricServiceRunning());

        when(sysConfig.getBoolean("artifactory.metrics.enabled", false)).thenReturn(true);
        when(sysConfig.get("artifactory.metrics.exclude", "")).thenReturn("");
        metricsService.onContextCreated();
        assertTrue(metricsService.isMetricServiceRunning());
        TestUtils.setField(ArtifactoryHome.get(), "sysConfig", sysConfigBackUp);
    }

    @Test
    public void testNoESUrlForFilebeat() {
        SysConfig sysConfig = mock(SysConfig.class);
        SysConfig sysConfigBackUp = TestUtils.getField(ArtifactoryHome.get(), "sysConfig", SysConfig.class);
        TestUtils.setField(ArtifactoryHome.get(), "sysConfig", sysConfig);
        String esUrl = metricsService.getESUrlFromSysConfig(sysConfig);
        assertNull(esUrl);
        TestUtils.setField(ArtifactoryHome.get(), "sysConfig", sysConfigBackUp);
    }

    @Test
    public void testESUrlForFilebeatWithSlash() {
        SysConfig sysConfig = mock(SysConfig.class);
        SysConfig sysConfigBackUp = TestUtils.getField(ArtifactoryHome.get(), "sysConfig", SysConfig.class);
        TestUtils.setField(ArtifactoryHome.get(), "sysConfig", sysConfig);

        when(sysConfig.get("shared.metrics.filebeat.elasticsearch.url"))
                .thenReturn(Optional.of("http://localhost:8082/elasticsearch/"));
        String esUrl = metricsService.getESUrlFromSysConfig(sysConfig);
        assertNotNull(esUrl);
        assertEquals("http://localhost:8082/elasticsearch/", esUrl);
        TestUtils.setField(ArtifactoryHome.get(), "sysConfig", sysConfigBackUp);
    }

    @Test
    public void testESUrlForFilebeatWithoutSlash() {
        SysConfig sysConfig = mock(SysConfig.class);
        SysConfig sysConfigBackUp = TestUtils.getField(ArtifactoryHome.get(), "sysConfig", SysConfig.class);
        TestUtils.setField(ArtifactoryHome.get(), "sysConfig", sysConfig);

        when(sysConfig.get("shared.metrics.filebeat.elasticsearch.url"))
                .thenReturn(Optional.of("http://localhost:8082/elasticsearch"));
        String esUrl = metricsService.getESUrlFromSysConfig(sysConfig);
        assertNotNull(esUrl);
        assertEquals("http://localhost:8082/elasticsearch/", esUrl);
        TestUtils.setField(ArtifactoryHome.get(), "sysConfig", sysConfigBackUp);
    }
}