package org.artifactory.security.token;

import org.artifactory.security.SimpleUser;
import org.artifactory.security.authentication.OauthAuthentication;
import org.artifactory.security.props.auth.model.AuthenticationModel;
import org.artifactory.security.props.auth.model.OauthModel;
import org.artifactory.security.providermgr.ArtifactoryCacheKey;
import org.artifactory.security.providermgr.ProviderMgr;
import org.mockito.Mockito;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.Test;

import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertTrue;

/**
 * @author Alexei Vainshtein
 */
public class ArtifactoryTokenProviderTest {

    @Test
    public void testGetToken() {
        authenticate();
        ArtifactoryTokenProvider tokenProvider = new ArtifactoryTokenProvider();
        ArtifactoryCacheKey cacheKey = Mockito.mock(ArtifactoryCacheKey.class);
        Mockito.when(cacheKey.getUser()).thenReturn("test");
        ProviderMgr providerMgr = Mockito.mock(ProviderMgr.class);
        AuthenticationModel oauthModel = new AuthenticationModel();
        oauthModel.setExpiresIn(10);
        oauthModel.setToken("token");
        Mockito.when(providerMgr.fetchAndStoreTokenFromProvider("test")).thenReturn(oauthModel);
        Mockito.when(cacheKey.getProviderMgr()).thenReturn(providerMgr);
        OauthModel token = tokenProvider.getToken(cacheKey);
        assertNotNull(token);
        assertTrue(token instanceof AuthenticationModel);
        assertEquals((int) ((AuthenticationModel) token).getExpiresIn(), 10);
        assertEquals(((AuthenticationModel) token).getToken(), "token");
    }

    @Test
    public void testGetTokenOauth() {
        OauthAuthentication oauthAuthentication = Mockito.mock(OauthAuthentication.class);
        SecurityContextHolder.getContext().setAuthentication(oauthAuthentication);
        OauthModel oauthModel = Mockito.mock(OauthModel.class);
        Mockito.when(oauthAuthentication.getModel()).thenReturn(oauthModel);
        ArtifactoryTokenProvider tokenProvider = new ArtifactoryTokenProvider();
        OauthModel token = tokenProvider.getToken(null);
        Assert.assertEquals(token, oauthModel);
    }

    private void authenticate() {
        Authentication authentication = Mockito.mock(Authentication.class);
        SecurityContextHolder.getContext().setAuthentication(authentication);
        when(authentication.getName()).thenReturn("test");
        SimpleUser simpleUser = Mockito.mock(SimpleUser.class);
        when(simpleUser.isTransientUser()).thenReturn(false);
        when(authentication.getPrincipal()).thenReturn(simpleUser);
    }

    @AfterMethod
    public void cleanup() {
        SecurityContextHolder.getContext().setAuthentication(null);
    }
}