package org.artifactory.storage.db.version.converter;

import org.artifactory.storage.db.version.ConversionPredicate;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Alexei Vainshtein
 */
public class AsyncDBSqlConditionalConverter extends ConditionalDBSqlConverter {

    private static final Logger log = LoggerFactory.getLogger(AsyncDBSqlConditionalConverter.class);
    private static final int NUM_OF_RETRIES = 3;

    public AsyncDBSqlConditionalConverter(String fromVersion, ConversionPredicate shouldConvert) {
        super(fromVersion, shouldConvert);
    }

    @Override
    public void convert(JdbcHelper jdbcHelper, DbType dbType) {
        try {
            convertWithRetries(jdbcHelper, dbType, 0);
        } catch (RuntimeException e) {
            log.error("Failed to convert {} for {} times due to:", fromVersion, NUM_OF_RETRIES, e);
        }
    }

    void convertWithRetries(JdbcHelper jdbcHelper, DbType dbType, int runs) {
        try {
            runs++;
            log.info("Starting attempt #{} of async conversion for {}", runs, fromVersion);
            super.convert(jdbcHelper, dbType);
            log.info("Conversion of {} finished successfully.", fromVersion);
        } catch (RuntimeException e) {
            log.error("Failed attempt #{} to convert {} due to:", fromVersion, e.getMessage());
            if (runs < NUM_OF_RETRIES) {
                convertWithRetries(jdbcHelper, dbType, runs);
                return;
            }
            log.debug("Failed attempt #{} to convert {} due to:", fromVersion, e.getMessage(), e);
            throw e;
        }
    }
}
