package org.artifactory.storage.db.conversion.version.v225;

import org.apache.commons.lang3.exception.ExceptionUtils;
import org.artifactory.storage.db.version.v225.V225LockingDBSqlConverter;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JFrogDataSource;
import org.jfrog.storage.JdbcHelper;
import org.junit.Assert;
import org.mockito.Mockito;
import org.testng.annotations.Test;

import java.sql.SQLException;

/**
 * Verify that @See org.artifactory.storage.db.version.v225.V225LockingDBSqlConverter is
 * converting only on POSTGRESQL.
 *
 * @author shaharz
 */
public class V225LockingDBSqlConverterTest {

    @Test
    public void testWithPostgreSql() throws Throwable {
        //Given
        V225LockingDBSqlConverter converter =
                new V225LockingDBSqlConverter("v225_alter_distributed_locks_set_unlogged");
        JdbcHelper jdbcHelper = Mockito.mock(JdbcHelper.class);
        JFrogDataSource dataSource = Mockito.mock(JFrogDataSource.class);
        Mockito.when(dataSource.getConnection()).thenThrow(SQLException.class);
        Mockito.when(jdbcHelper.getDataSource()).thenReturn(dataSource);

        try {
            //When
            converter.convert(jdbcHelper, DbType.POSTGRESQL);
            //Then
            Assert.fail();
        } catch (Exception e) {
            //Then
            // If we got here with SQLException we are OK
            // If we got into the super converter it means that POSTGRESQL verification works
            // Our mocked JFrogDataSource.getConnection() throw SQLException

            Assert.assertTrue(ExceptionUtils.getRootCause(e) instanceof SQLException);
        }
    }

    @Test
    public void testWithNonePostgreSql() throws Throwable {
        //Given
        V225LockingDBSqlConverter converter =
                new V225LockingDBSqlConverter("v225_alter_distributed_locks_set_unlogged");
        JdbcHelper jdbcHelper = Mockito.mock(JdbcHelper.class);
        JFrogDataSource dataSource = Mockito.mock(JFrogDataSource.class);
        Mockito.when(dataSource.getConnection()).thenThrow(new SQLException("Fake SQLException"));
        Mockito.when(jdbcHelper.getDataSource()).thenReturn(dataSource);

        try {
            // If we pass the convert with no exception, we are ok, meaning the call was not
            // proceeded to the super class convert, the POSTGRESQL condition worked.

            //When
            converter.convert(jdbcHelper, DbType.ORACLE);

            //When
            converter.convert(jdbcHelper, DbType.MYSQL);

            //When
            converter.convert(jdbcHelper, DbType.MSSQL);

            //When
            converter.convert(jdbcHelper, DbType.DERBY);

            //When
            converter.convert(jdbcHelper, DbType.MARIADB);
        } catch (Exception e) {
            //Then
            Assert.fail();
        }
    }
}
