package org.artifactory.webapp.servlet.util;

import org.artifactory.webapp.servlet.authentication.ArtifactoryAccessTokenAuthenticationFilter;
import org.artifactory.webapp.servlet.authentication.ArtifactoryAuthenticationFilter;
import org.artifactory.webapp.servlet.authentication.ArtifactoryBasicAuthenticationFilter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.authentication.BadCredentialsException;

import javax.servlet.ServletRequest;

/**
 * @author Mor Merhav
 */
public final class ServletUtils {

    private static final Logger log = LoggerFactory.getLogger(ServletUtils.class);

    private ServletUtils() {

    }

    /**
     * @return login identifier
     */
    public static String getLoginIdentifier(ServletRequest servletRequest, ArtifactoryAuthenticationFilter filter) {
        String loginIdentifier = filter.getCacheKey(servletRequest);
        try {
            // fetch context LoginIdentifier
            loginIdentifier = filter.getLoginIdentifier(servletRequest);
            if (filter instanceof ArtifactoryAccessTokenAuthenticationFilter ||
                    filter instanceof ArtifactoryBasicAuthenticationFilter) {
                log.debug("Found loginIdentifier {}", loginIdentifier);
            }
        } catch (BadCredentialsException e) {
            log.debug("Resolving uses access details has failed, {}", e.getMessage());
            if (loginIdentifier == null) { loginIdentifier = ""; }
        }
        return loginIdentifier;
    }
}
