/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.webapp.servlet.authentication;

import org.apache.commons.codec.binary.Base64;
import org.artifactory.security.ldap.LdapRealmAwareAuthentication;
import org.artifactory.webapp.servlet.RequestUtils;
import org.junit.Assert;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import javax.servlet.http.HttpServletRequest;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

/**
 * @author dudim
 */
public class ArtifactoryBasicAuthenticationFilterTest {

    @Mock
    private ArtifactoryBasicAuthenticationFilter artifactoryBasicAuthenticationFilter;
    @Mock
    private LdapRealmAwareAuthentication ldapRealmAwareAuthentication;

    @BeforeMethod
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }

    /**
     * RTFACT-15967
     * BasicAuthenticationFilter is used when authenticating LDAP users,
     * when first time authenticate success, the user creds is stored in AccessFilter:nonUiAuthCache,
     * when doing second time authentication, we check if user creds already exist,
     * this check is failed due case sensitive issue.
     */
    @Test
    public void testRequiresReAuthentication() throws Exception {
        when(artifactoryBasicAuthenticationFilter.acceptFilter(any())).thenReturn(true);
        when(ldapRealmAwareAuthentication.getPrincipal()).thenReturn("dudim");
        when(artifactoryBasicAuthenticationFilter.requiresReAuthentication(any(), any())).thenCallRealMethod();
        HttpServletRequest httpRequestMock = Mockito.mock(HttpServletRequest.class);
        String base64UsernamePassword = Base64.encodeBase64String("DUDIM:password".getBytes());
        when(httpRequestMock.getHeader(RequestUtils.AUTHORIZATION_HEADER))
                .thenReturn("Basic " + base64UsernamePassword);
        Assert.assertFalse(
                artifactoryBasicAuthenticationFilter
                        .requiresReAuthentication(httpRequestMock, ldapRealmAwareAuthentication));
    }
}